package org.artifactory.ph;

import org.apache.http.HttpStatus;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.api.security.SecurityService;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.service.PackageSecurityService;
import org.artifactory.repo.RepoPathFactory;

import java.util.concurrent.Callable;

public class PackageSecurityServiceImpl implements PackageSecurityService {

    private final AuthorizationService authorizationService;
    private final SecurityService securityService;

    public PackageSecurityServiceImpl(AuthorizationService authorizationService,
            SecurityService securityService) {
        this.authorizationService = authorizationService;
        this.securityService = securityService;
    }

    @Override
    public boolean canRead(String repoKey, String path) {
        return authorizationService.canRead(RepoPathFactory.create(repoKey, path));
    }

    @Override
    public boolean canWrite(String repoKey, String path) {
        return authorizationService.canDeploy(RepoPathFactory.create(repoKey, path));
    }

    @Override
    public boolean canAnnotate(String repoKey, String path) {
        return authorizationService.canAnnotate(RepoPathFactory.create(repoKey, path));
    }

    @Override
    public boolean canDelete(String repoKey, String path) {
        return authorizationService.canDelete(RepoPathFactory.create(repoKey, path));
    }

    @Override
    public boolean canManage(String repoKey, String path) {
        return authorizationService.canManage(RepoPathFactory.create(repoKey, path));
    }

    @Override
    public <V> V callAsSystem(Callable<V> callable) throws PackageException {
        try {
            return securityService.callAsSystem(callable);
        } catch (PackageException e) {
            throw e;
        } catch (Exception e) {
            throw new PackageException(HttpStatus.SC_INTERNAL_SERVER_ERROR, e.getMessage(), e.getMessage(), e);
        }
    }

    @Override
    public boolean isAnonymous() {
        return authorizationService.isAnonymous();
    }

    @Override
    public boolean isAnonAccessEnabled() {
        return authorizationService.isAnonAccessEnabled();
    }
}
