package org.artifactory.event;

import org.artifactory.storage.StorageException;
import org.artifactory.storage.db.event.dao.EventsDao;
import org.artifactory.storage.db.event.service.EventsServiceImpl;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.sql.SQLException;

import static org.fest.assertions.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@Test
public class EventServiceTest {

    @InjectMocks
    EventsServiceImpl eventsService;

    @Mock
    EventsDao eventsDao;

    @BeforeMethod
    public void init() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void testCountEventsRangeByRepoKey() throws SQLException {
        when(eventsDao.countRangeEventsByRepoPath(1515653250000L,1515653250004L, "repo")).thenReturn(3);
        int count = eventsService.countEventsRangeByRepoKey(1515653250000L, 1515653250004L, "repo" );
        assertThat(count).isEqualTo(3);
    }

    @Test
    public void testCountEventsRangeByRepoKeyWithFailure() throws SQLException{
        try {
            when(eventsDao.countRangeEventsByRepoPath(anyLong(), anyLong(), anyString()))
                    .thenThrow(new SQLException("bad"));
            int count = eventsService.countEventsRangeByRepoKey(1515653250000L, 1515653250004L, "repo" );
        }catch (StorageException e){
            assertThat(e.getMessage()).contains("Failed to get event range on repo");
        }
    }
}
