/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.descriptor.repo;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotEquals;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

/**
 * Tests the VirtualRepoDescriptor.
 *
 * @author Yossi Shaul
 */
@Test
public class VirtualRepoDescriptorTest {

    private final LocalRepoDescriptor local1 = new LocalRepoDescriptor();
    private final LocalRepoDescriptor local2 = new LocalRepoDescriptor();

    @BeforeMethod
    public void initTestRepos() {
        local1.setKey("local1");
        local2.setKey("local2");
    }

    public void defaultConstructor() {
        VirtualRepoDescriptor virtualRepo = new VirtualRepoDescriptor();
        assertNull(virtualRepo.getKey());
        assertNull(virtualRepo.getRepoLayout());
        assertNull(virtualRepo.getKeyPair());
        assertTrue(virtualRepo.getRepositories().isEmpty());
        assertNotEquals(virtualRepo.getType(), RepoType.NuGet, "NuGet should be off by default.");
        assertEquals(virtualRepo.getDockerApiVersion(), DockerApiVersion.V2,
                "Docker API version should be V2 by default.");
    }

    public void virtualWithDuplicateLocalRepo() {
        VirtualRepoDescriptor virtual = new VirtualRepoDescriptor();
        virtual.setKey("virtual");
        assertEquals(virtual.getRepositories().size(), 0, "Expecting 0 repositories");
        List<RepoDescriptor> reposList = Arrays.asList(local1, local1, local2);
        assertEquals(reposList.size(), 3, "Expecting 3 repositories, 'local1' duplicate");
        virtual.setRepositories(reposList);
        assertEquals(virtual.getRepositories().size(), 2, "Expecting 2 repositories - without duplication");
        virtual.setRepositories(Collections.emptyList());
        assertEquals(virtual.getRepositories().size(), 0, "Expecting 0 repositories");
    }
}
