package org.artifactory.core.api.ph.virtual.resolution;

import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.model.PackageDownloadContext;

import javax.validation.constraints.NotNull;

/**
 * Basic virtual repository controller logic
 */
public interface PackageVirtualController<T extends PackageMetadataRow> extends PackageVirtualMerger<T>,
        PackageVirtualResolver {

    /**
     * The method is being called by the virtual visitor framework,
     * to support virtual repo that contains repositories with different layouts
     *
     * @param packageRepo    repository to translate )
     * @param requestContext request context
     * @return new modified request context if necessary,
     * retyrn original request context if there is bo need for modification/translation
     */
    default PackageRequestContext translateSubRepositoryContext(PackageRepo packageRepo,
            PackageRequestContext requestContext) throws PackageException {
        return new PackageDownloadContext(packageRepo.getKey(), requestContext.getPath());
    }

    /**
     * This method is being called by the virtual visitor framework.
     * The method allows the virtual visitor framework to determinate whether to use the resolution strategy or the
     * merging strategy
     *
     * The method should never return null;
     *
     * @param packageRequestContext request context for the artifact
     * @return preferred strategy
     */
    @NotNull
    default VirtualResolutionStrategyType getVirtualResolutionStrategy(PackageRequestContext packageRequestContext) {
        return VirtualResolutionStrategyType.RESOLVER;
    }

    /**
     * The method is being called by the virtual visitor framework,
     * in some cases the package layer persist the metadata cache in different names,
     * this method allows to override the request context info
     *
     * @param context request context
     * @return new modified request context if necessary, if not return the original request context
     */
    default PackageRequestContext getCachePath(PackageRequestContext context) {
        return context;
    }
}
