package org.artifactory.storage.db.binstore.service;

import lombok.AllArgsConstructor;
import org.artifactory.storage.binstore.service.InternalBinaryService;
import org.artifactory.storage.db.DbService;
import org.artifactory.storage.db.remotebintask.RemoteBinaryTaskDao;
import org.jfrog.storage.binstore.providers.base.PersistedRemoteTask;
import org.jfrog.storage.binstore.providers.base.PersistedTaskManager;

import java.sql.SQLException;
import java.util.List;

/**
 * @author barh
 */
@AllArgsConstructor
public class PersistedTaskManagerImpl implements PersistedTaskManager<PersistedRemoteTask> {

    RemoteBinaryTaskDao remoteBinaryTaskDao;
    InternalBinaryService binaryService;
    DbService dbService;

    @Override
    public List<PersistedRemoteTask> getTasks(long batchSize, int maxExecutionsCount,
            long retryDelayMs,
            long maximumExecTimeMs) throws SQLException {
        return binaryService
                .getNextRemoteTaskFromDataDb(batchSize, maxExecutionsCount, retryDelayMs, maximumExecTimeMs);
    }

    @Override
    public void put(PersistedRemoteTask task) throws SQLException {
        binaryService.offerRemoteTask(task);
    }

    @Override
    public void updateError(PersistedRemoteTask task) throws SQLException {
        remoteBinaryTaskDao.deAllocateById(task.getId());
    }

    @Override
    public void delete(PersistedRemoteTask task) throws SQLException {
        remoteBinaryTaskDao.remove(task.getId());
    }
}
