/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.p2.P2Repo;
import org.artifactory.addon.webstart.ArtifactWebstartAddon;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.property.PropertySet;
import org.artifactory.descriptor.replication.LocalReplicationDescriptor;
import org.artifactory.descriptor.replication.RemoteReplicationDescriptor;
import org.artifactory.descriptor.repo.BowerConfiguration;
import org.artifactory.descriptor.repo.CdnRedirectRepoConfig;
import org.artifactory.descriptor.repo.CocoaPodsConfiguration;
import org.artifactory.descriptor.repo.ComposerConfiguration;
import org.artifactory.descriptor.repo.DockerApiVersion;
import org.artifactory.descriptor.repo.DownloadRedirectRepoConfig;
import org.artifactory.descriptor.repo.ExternalDependenciesConfig;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.GitConfiguration;
import org.artifactory.descriptor.repo.HelmConfiguration;
import org.artifactory.descriptor.repo.HttpRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.NuGetConfiguration;
import org.artifactory.descriptor.repo.ProxyDescriptor;
import org.artifactory.descriptor.repo.PypiConfiguration;
import org.artifactory.descriptor.repo.RealRepoDescriptor;
import org.artifactory.descriptor.repo.RepoBaseDescriptor;
import org.artifactory.descriptor.repo.RepoDescriptor;
import org.artifactory.descriptor.repo.RepoLayout;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.descriptor.repo.ReverseProxyDescriptor;
import org.artifactory.descriptor.repo.ReverseProxyRepoConfig;
import org.artifactory.descriptor.repo.TerraformConfiguration;
import org.artifactory.descriptor.repo.VcsConfiguration;
import org.artifactory.descriptor.repo.VcsGitConfiguration;
import org.artifactory.descriptor.repo.VirtualRepoDescriptor;
import org.artifactory.descriptor.repo.VirtualRepoResolver;
import org.artifactory.descriptor.repo.XrayRepoConfig;
import org.artifactory.descriptor.repo.releasebundle.ReleaseBundlesRepoDescriptor;
import org.artifactory.repo.FederatedMemberConfiguration;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.ui.rest.model.admin.configuration.propertysets.PropertySetNameModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.AdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.BasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.CdnRedirectRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.DownloadRedirectRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.GeneralRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepoAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederationConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.MirrorMemberConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.releasebundles.ReleaseBundlesAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.releasebundles.ReleaseBundlesBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.releasebundles.ReleaseBundlesRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteCacheRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteNetworkRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.replication.local.LocalReplicationConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.replication.remote.RemoteReplicationConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.AlpineTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.BowerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.BuildInfoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CargoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ChefTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CocoaPodsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ComposerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ConanTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CondaTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CranTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DebTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DockerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GemsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GenericTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GitLfsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GradleTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.HelmTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.IvyTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.MavenTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.NpmTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.NugetTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.OpkgTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.P2TypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.PubTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.PuppetTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.PypiTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ReleaseBundlesRepoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.SbtTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.SwiftTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TerraformBackendTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TerraformTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.VagrantTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.VcsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.YumTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualSelectedRepository;
import org.artifactory.ui.rest.model.admin.configuration.repository.xray.XrayRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.reverseProxy.ReverseProxyRepoModel;
import org.artifactory.util.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.Nullable;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ConcurrentMap;
import java.util.stream.Collectors;

/**
 * Utility class for converting descriptor to model
 *
 * @author Aviad Shikloshi
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class RepoConfigModelBuilder {
    private static final Logger log = LoggerFactory.getLogger(RepoConfigModelBuilder.class);

    private CentralConfigService centralConfig;

    private AddonsManager addonsManager;

    /**
     * Populate model configuration from local repository descriptor
     */
    public void populateLocalDescriptorValuesToModel(LocalRepoDescriptor descriptor,
            LocalRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(descriptor);
        LocalBasicRepositoryConfigModel basic = createLocalBasicConfig(descriptor);
        LocalAdvancedRepositoryConfigModel advanced = createLocalAdvancedConfig(descriptor);
        TypeSpecificConfigModel typeSpecific = createLocalTypeSpecific(descriptor.getType(), descriptor);

        List<LocalReplicationDescriptor> replicationDescriptors = centralConfig.getDescriptor()
                .getMultiLocalReplications(descriptor.getKey());
        List<LocalReplicationConfigModel> replications = replicationDescriptors.stream()
                .map(this::createLocalReplicationConfig)
                .collect(Collectors.toList());

        model.setGeneral(general);
        model.setBasic(basic);
        model.setAdvanced(advanced);
        model.setTypeSpecific(typeSpecific);
        model.setReplications(replications);
    }

    /**
     * Populate model configuration from federated repository descriptor
     */
    public void populateFederatedDescriptorValuesToModel(FederatedRepoDescriptor descriptor,
            FederatedRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(descriptor);
        LocalBasicRepositoryConfigModel basic = createLocalBasicConfig(descriptor);
        FederatedRepoAdvancedRepositoryConfigModel advanced = createFederatedRepoAdvancedConfig(descriptor);
        TypeSpecificConfigModel typeSpecific = createLocalTypeSpecific(descriptor.getType(), descriptor);

        List<FederatedMemberConfiguration> federationMembers = centralConfig.getDescriptor()
                .getFederationMembers(descriptor.getKey());
        List<MirrorMemberConfigModel> members = federationMembers.stream()
                .map(this::toMemberConfigModel)
                .collect(Collectors.toList());
        FederationConfigModel federationConfigModel = new FederationConfigModel();
        federationConfigModel.setMirrors(members);

        model.setFederation(federationConfigModel);

        model.setGeneral(general);
        model.setBasic(basic);
        model.setAdvanced(advanced);
        model.setTypeSpecific(typeSpecific);
    }

    private MirrorMemberConfigModel toMemberConfigModel(FederatedMemberConfiguration federatedMemberConfiguration) {
        MirrorMemberConfigModel memberConfigModel = new MirrorMemberConfigModel();
        memberConfigModel.setUrl(federatedMemberConfiguration.getUrl());
        memberConfigModel.setEnabled(federatedMemberConfiguration.getEnabled());
        return memberConfigModel;
    }

    /**
     * Populate model configuration from remote repository descriptor
     */
    public void populateRemoteRepositoryConfigValuesToModel(HttpRepoDescriptor descriptor,
            RemoteRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(descriptor);
        model.setGeneral(general);

        RemoteBasicRepositoryConfigModel basic = createRemoteBasicConfig(descriptor);
        model.setBasic(basic);

        RemoteReplicationDescriptor replicationDescriptor = centralConfig.getDescriptor().getRemoteReplication(
                descriptor.getKey());
        if (replicationDescriptor != null) {
            RemoteReplicationConfigModel replication = createRemoteReplicationConfigModel(replicationDescriptor);
            model.setReplications(Lists.newArrayList(replication));
        }
        RemoteAdvancedRepositoryConfigModel advanced = createRemoteAdvancedConfig(descriptor);
        model.setAdvanced(advanced);
        TypeSpecificConfigModel typeSpecific = createRemoteTypeSpecific(descriptor.getType(), descriptor);
        model.setTypeSpecific(typeSpecific);
    }

    /**
     * Populate model configuration from virtual repository descriptor
     */
    public void populateVirtualRepositoryConfigValuesToModel(VirtualRepoDescriptor descriptor,
            VirtualRepositoryConfigModel model) {
        // General
        GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        general.setRepoKey(descriptor.getKey());
        model.setGeneral(general);

        // Basic
        VirtualBasicRepositoryConfigModel basic = new VirtualBasicRepositoryConfigModel();
        Optional.ofNullable(descriptor.getRepoLayout()).ifPresent(layout -> basic.setLayout(layout.getName()));
        Optional.ofNullable(descriptor.getPrimaryKeyPairDescriptor())
                .ifPresent(keyPair -> {
                    basic.setKeypair(keyPair.getKey());
                    basic.setPrimaryKeyPair(keyPair.getKey());
                });
        Optional.ofNullable(descriptor.getSecondaryKeyPairDescriptor())
                .ifPresent(keyPair -> basic.setSecondaryKeyPair(keyPair.getKey()));
        basic.setPublicDescription(descriptor.getDescription());
        basic.setInternalDescription(descriptor.getNotes());
        basic.setExcludesPattern(descriptor.getExcludesPattern());
        basic.setIncludesPattern(descriptor.getIncludesPattern());
        List<RepoDescriptor> repositories = descriptor.getRepositories();
        VirtualRepoResolver resolver = new VirtualRepoResolver(descriptor);
        basic.setResolvedRepositories(
                resolver.getOrderedRepos().stream().map(VirtualSelectedRepository::new).collect(Collectors.toList()));
        basic.setSelectedRepositories(
                repositories.stream().map(VirtualSelectedRepository::new).collect(Collectors.toList()));
        Optional.ofNullable(descriptor.getDefaultDeploymentRepo())
                .ifPresent(localRepoDescriptor -> basic.setDefaultDeploymentRepo(localRepoDescriptor.getKey()));
        model.setBasic(basic);

        // Advanced
        VirtualAdvancedRepositoryConfigModel advanced = new VirtualAdvancedRepositoryConfigModel();
        advanced.setRetrieveRemoteArtifacts(descriptor.isArtifactoryRequestsCanRetrieveRemoteArtifacts());
        addReverseProxyConfig(advanced, descriptor.getKey());
        advanced.setHideUnauthorizedResources(descriptor.isHideUnauthorizedResources());
        model.setAdvanced(advanced);

        // Type specific
        TypeSpecificConfigModel typeSpecific = createVirtualTypeSpecific(descriptor.getType(), descriptor);
        model.setTypeSpecific(typeSpecific);
    }

    /**
     * Populate model configuration from local repository descriptor
     */
    public void populateReleaseBundlesDescriptorValuesToModel(ReleaseBundlesRepoDescriptor descriptor,
            ReleaseBundlesRepositoryConfigModel model) {
        GeneralRepositoryConfigModel general = createGeneralConfig(descriptor);
        ReleaseBundlesBasicRepositoryConfigModel basic = createReleaseBundlesBasicConfig(descriptor);
        ReleaseBundlesAdvancedRepositoryConfigModel advanced = createReleaseBundlesAdvancedConfig(descriptor);
        ReleaseBundlesRepoTypeSpecificConfigModel typeSpecific = new ReleaseBundlesRepoTypeSpecificConfigModel();
        model.setGeneral(general);
        model.setBasic(basic);
        model.setAdvanced(advanced);
        model.setTypeSpecific(typeSpecific);
    }

    private LocalReplicationConfigModel createLocalReplicationConfig(LocalReplicationDescriptor replicationDescriptor) {
        LocalReplicationConfigModel replication = new LocalReplicationConfigModel();
        replication.setUrl(replicationDescriptor.getUrl());
        replication.setCronExp(replicationDescriptor.getCronExp());
        replication.setEnableEventReplication(replicationDescriptor.isEnableEventReplication());
        replication.setEnabled(replicationDescriptor.isEnabled());

        ProxyDescriptor proxyDescriptor = replicationDescriptor.getProxy();
        if (proxyDescriptor != null) {
            replication.setProxy(proxyDescriptor.getKey());
        }
        replication.setSocketTimeout(replicationDescriptor.getSocketTimeoutMillis());
        replication.setSyncDeletes(replicationDescriptor.isSyncDeletes());
        replication.setSyncProperties(replicationDescriptor.isSyncProperties());
        replication.setSyncStatistics(replicationDescriptor.isSyncStatistics());
        replication.setCheckBinaryExistenceInFilestore(replicationDescriptor.isCheckBinaryExistenceInFilestore());
        replication.setUsername(replicationDescriptor.getUsername());
        replication.setPassword(replicationDescriptor.getPassword());
        replication.setIncludePathPrefixPattern(replicationDescriptor.getIncludePathPrefixPattern());
        replication.setExcludePathPrefixPattern(replicationDescriptor.getExcludePathPrefixPattern());
        return replication;
    }

    private RemoteReplicationConfigModel createRemoteReplicationConfigModel(
            RemoteReplicationDescriptor replicationDescriptor) {
        RemoteReplicationConfigModel replication = new RemoteReplicationConfigModel();
        replication.setEnabled(replicationDescriptor.isEnabled());
        replication.setCronExp(replicationDescriptor.getCronExp());
        replication.setSyncDeletes(replicationDescriptor.isSyncDeletes());
        replication.setSyncProperties(replicationDescriptor.isSyncProperties());
        replication.setEnableEventReplication(replicationDescriptor.isEnableEventReplication());
        replication.setIncludePathPrefixPattern(replicationDescriptor.getIncludePathPrefixPattern());
        replication.setExcludePathPrefixPattern(replicationDescriptor.getExcludePathPrefixPattern());
        return replication;
    }

    private GeneralRepositoryConfigModel createGeneralConfig(RepoBaseDescriptor descriptor) {
        GeneralRepositoryConfigModel general = new GeneralRepositoryConfigModel();
        general.setRepoKey(descriptor.getKey());
        return general;
    }

    private LocalBasicRepositoryConfigModel createLocalBasicConfig(LocalRepoDescriptor descriptor) {
        LocalBasicRepositoryConfigModel basic = new LocalBasicRepositoryConfigModel();
        addSharedBasicConfigModel(basic, descriptor);
        basic.setXrayConfig(getXrayConfig(descriptor.getXrayConfig()));
        return basic;
    }

    private RemoteBasicRepositoryConfigModel createRemoteBasicConfig(HttpRepoDescriptor descriptor) {
        RemoteBasicRepositoryConfigModel basic = new RemoteBasicRepositoryConfigModel();
        addSharedBasicConfigModel(basic, descriptor);
        basic.setUrl(descriptor.getUrl());
        basic.setXrayConfig(getXrayConfig(descriptor.getXrayConfig()));
        basic.setContentSynchronisation(descriptor.getContentSynchronisation());
        RepoLayout remoteRepoLayout = descriptor.getRemoteRepoLayout();
        if (remoteRepoLayout != null) {
            basic.setRemoteLayoutMapping(remoteRepoLayout.getName());
        }
        basic.setOffline(descriptor.isOffline());
        return basic;
    }

    private ReleaseBundlesBasicRepositoryConfigModel createReleaseBundlesBasicConfig(
            ReleaseBundlesRepoDescriptor descriptor) {
        ReleaseBundlesBasicRepositoryConfigModel basic = new ReleaseBundlesBasicRepositoryConfigModel();
        addSharedBasicConfigModel(basic, descriptor);
        return basic;
    }

    private void addSharedBasicConfigModel(BasicRepositoryConfigModel basic, RepoDescriptor descriptor) {
        basic.setPublicDescription(descriptor.getDescription());
        basic.setInternalDescription(descriptor.getNotes());
        basic.setIncludesPattern(descriptor.getIncludesPattern());
        basic.setExcludesPattern(descriptor.getExcludesPattern());
        Optional.ofNullable(descriptor.getRepoLayout()).ifPresent(repoLayout -> basic.setLayout(repoLayout.getName()));
        Optional.ofNullable(descriptor.getPrimaryKeyPairDescriptor())
                .ifPresent(keyPair -> {
                    basic.setPrimaryKeyPair(keyPair.getKey());
                    basic.setKeypair(keyPair.getKey());
                });
        Optional.ofNullable(descriptor.getSecondaryKeyPairDescriptor())
                .ifPresent(keyPair -> basic.setSecondaryKeyPair(keyPair.getKey()));
    }

    private LocalAdvancedRepositoryConfigModel createLocalAdvancedConfig(LocalRepoDescriptor descriptor) {
        LocalAdvancedRepositoryConfigModel advanced = new LocalAdvancedRepositoryConfigModel();
        populateRepoAdvancedConfig(descriptor, advanced);
        return advanced;
    }

    private void populateRepoAdvancedConfig(LocalRepoDescriptor descriptor,
            LocalAdvancedRepositoryConfigModel advanced) {
        addSharedAdvancedConfigModel(advanced, descriptor);
        advanced.setDownloadRedirectConfig(getDownloadRedirectConfig(descriptor.getDownloadRedirectConfig()));

        if (descriptor.getCdnRedirectRepoConfig() != null) {
            final CdnRedirectRepoConfigModel cdnRedirectRepoConfigModel = new CdnRedirectRepoConfigModel();
            cdnRedirectRepoConfigModel.setEnabled(descriptor.getCdnRedirectRepoConfig().isEnabled());
            advanced.setCdnRedirectRepoConfig(cdnRedirectRepoConfigModel);
        }
        advanced.setPriorityResolution(descriptor.isPriorityResolution());
    }

    private FederatedRepoAdvancedRepositoryConfigModel createFederatedRepoAdvancedConfig(
            FederatedRepoDescriptor descriptor) {
        FederatedRepoAdvancedRepositoryConfigModel advanced = new FederatedRepoAdvancedRepositoryConfigModel();
        populateRepoAdvancedConfig(descriptor, advanced);
        if (descriptor.getProxy() != null) {
            advanced.setProxy(descriptor.getProxy().getKey());
        }
        return advanced;
    }


    private RemoteAdvancedRepositoryConfigModel createRemoteAdvancedConfig(HttpRepoDescriptor descriptor) {
        RemoteAdvancedRepositoryConfigModel advanced = new RemoteAdvancedRepositoryConfigModel();
        addSharedAdvancedConfigModel(advanced, descriptor);
        advanced.setDownloadRedirectConfig(getDownloadRedirectConfig(descriptor.getDownloadRedirectConfig()));
        advanced.setPriorityResolution(descriptor.isPriorityResolution());
        advanced.setSignedUrlTtl(descriptor.getSignedUrlTtl());
        RemoteNetworkRepositoryConfigModel networkModel = createNetworkConfig(descriptor);
        advanced.setNetwork(networkModel);
        RemoteCacheRepositoryConfigModel cacheConfig = createCacheConfig(descriptor);
        advanced.setCache(cacheConfig);
        advanced.setStoreArtifactsLocally(descriptor.isStoreArtifactsLocally());
        advanced.setSynchronizeArtifactProperties(descriptor.isSynchronizeProperties());
        advanced.setHardFail(descriptor.isHardFail());
        advanced.setQueryParams(descriptor.getQueryParams());
        advanced.setShareConfiguration(descriptor.isShareConfiguration());
        advanced.setBlockMismatchingMimeTypes(descriptor.isBlockMismatchingMimeTypes());
        advanced.setBypassHeadRequests(descriptor.isBypassHeadRequests());
        advanced.setDisableUrlNormalization(descriptor.isDisableUrlNormalization());
        advanced.setPropagateQueryParams(descriptor.isPropagateQueryParams());
        final CdnRedirectRepoConfig cdnRedirectRepoConfig = descriptor.getCdnRedirectRepoConfig();
        if (cdnRedirectRepoConfig != null) {
            CdnRedirectRepoConfigModel cdnRedirectRepoConfigModel = new CdnRedirectRepoConfigModel();
            cdnRedirectRepoConfigModel.setEnabled(cdnRedirectRepoConfig.isEnabled());
            advanced.setCdnRedirectRepoConfig(cdnRedirectRepoConfigModel);
        }
        if (StringUtils.isNotBlank(descriptor.getMismatchingMimeTypesOverrideList())) {
            advanced.setMismatchingMimeTypesOverrideList(
                    Lists.newArrayList(descriptor.getMismatchingMimeTypesOverrideList().split(",")));
        }
        return advanced;
    }

    private ReleaseBundlesAdvancedRepositoryConfigModel createReleaseBundlesAdvancedConfig(
            LocalRepoDescriptor descriptor) {
        ReleaseBundlesAdvancedRepositoryConfigModel advanced = new ReleaseBundlesAdvancedRepositoryConfigModel();
        addSharedAdvancedConfigModel(advanced, descriptor);
        return advanced;
    }

    private void addSharedAdvancedConfigModel(AdvancedRepositoryConfigModel advanced, RealRepoDescriptor descriptor) {
        advanced.setAllowContentBrowsing(descriptor.isArchiveBrowsingEnabled());
        advanced.setBlackedOut(descriptor.isBlackedOut());
        advanced.setSignedUrlTtl(descriptor.getSignedUrlTtl());
        List<PropertySet> propertySetsList = descriptor.getPropertySets();
        List<PropertySetNameModel> propertySetNameModelList = collectPropertySets(propertySetsList);
        advanced.setPropertySets(propertySetNameModelList);
        addReverseProxyConfig(advanced, descriptor.getKey());
    }

    RemoteCacheRepositoryConfigModel createCacheConfig(HttpRepoDescriptor descriptor) {
        RemoteCacheRepositoryConfigModel cacheConfig = new RemoteCacheRepositoryConfigModel();
        cacheConfig.setKeepUnusedArtifactsHours(descriptor.getUnusedArtifactsCleanupPeriodHours());
        cacheConfig.setRetrievalCachePeriodSecs(descriptor.getRetrievalCachePeriodSecs());
        cacheConfig.setAssumedOfflineLimitSecs(descriptor.getAssumedOfflinePeriodSecs());
        cacheConfig.setMissedRetrievalCachePeriodSecs(descriptor.getMissedRetrievalCachePeriodSecs());
        cacheConfig.setMetadataRetrievalTimeoutSecs(descriptor.getMetadataRetrievalTimeoutSecs());
        return cacheConfig;
    }

    private RemoteNetworkRepositoryConfigModel createNetworkConfig(HttpRepoDescriptor descriptor) {
        RemoteNetworkRepositoryConfigModel networkModel = new RemoteNetworkRepositoryConfigModel();
        if (descriptor.getProxy() != null) {
            networkModel.setProxy(descriptor.getProxy().getKey());
        }
        networkModel.setLocalAddress(descriptor.getLocalAddress());
        networkModel.setUsername(descriptor.getUsername());
        networkModel.setPassword(descriptor.getPassword());
        networkModel.setSocketTimeout(descriptor.getSocketTimeoutMillis());
        networkModel.setLenientHostAuth(descriptor.isAllowAnyHostAuth());
        networkModel.setCookieManagement(descriptor.isEnableCookieManagement());
        networkModel.setSyncProperties(descriptor.isSynchronizeProperties());
        String clientTleCertificate = descriptor.getClientTlsCertificate();
        if (StringUtils.isNotBlank(clientTleCertificate) &&
                clientTleCertificate.contains(ArtifactWebstartAddon.SSL_CERT_ALIAS_PREFIX)) {
            clientTleCertificate = clientTleCertificate.substring(ArtifactWebstartAddon.SSL_CERT_ALIAS_PREFIX.length());
        }
        networkModel.setSelectedInstalledCertificate(clientTleCertificate);
        ArtifactWebstartAddon webstartAddon = addonsManager.addonByType(ArtifactWebstartAddon.class);
        List<String> certAliases = webstartAddon.getSslCertNames();
        if (CollectionUtils.notNullOrEmpty(certAliases)) {
            networkModel.setInstalledCertificatesList(certAliases);
        }
        return networkModel;
    }

    private TypeSpecificConfigModel createLocalTypeSpecific(RepoType type, LocalRepoDescriptor descriptor) {
        MavenTypeSpecificConfigModel mavenModel = null;
        TypeSpecificConfigModel model = null;
        switch (type) {
            case Bower:
                model = new BowerTypeSpecificConfigModel();
                break;
            case Cargo:
                CargoTypeSpecificConfigModel cargo = new CargoTypeSpecificConfigModel();
                cargo.setAnonymousAccess(descriptor.isCargoAnonymousAccess());
                model = cargo;
                break;
            case Chef:
                model = new ChefTypeSpecificConfigModel();
                break;
            case CocoaPods:
                model = new CocoaPodsTypeSpecificConfigModel();
                break;
            case Composer:
                ComposerTypeSpecificConfigModel composer = new ComposerTypeSpecificConfigModel();
                composer.setEnableComposerV1Indexing(descriptor.isEnableComposerV1Indexing());
                model = composer;
                break;
            case Conan:
                ConanTypeSpecificConfigModel conanType = new ConanTypeSpecificConfigModel();
                populateSharedConanValues(conanType, descriptor);
                model = conanType;
                break;
            case Docker:
                DockerTypeSpecificConfigModel dockerType = new DockerTypeSpecificConfigModel();
                DockerApiVersion dockerApiVersion = descriptor.getDockerApiVersion();
                if (dockerApiVersion != null) {
                    dockerType.setDockerApiVersion(dockerApiVersion);
                }
                dockerType.setMaxUniqueTags(descriptor.getMaxUniqueTags());
                dockerType.setBlockPushingSchema1(descriptor.isBlockPushingSchema1());
                dockerType.setDockerTagRetention(descriptor.getDockerTagRetention());
                model = dockerType;
                break;
            case Helm:
                model = new HelmTypeSpecificConfigModel();
                break;
            case Alpine:
                model = new AlpineTypeSpecificConfigModel();
                break;
            case Go:
                model = new GoTypeSpecificConfigModel();
                break;
            case CRAN:
                model = new CranTypeSpecificConfigModel();
                break;
            case Conda:
                model = new CondaTypeSpecificConfigModel();
                break;
            case NuGet:
                NugetTypeSpecificConfigModel nugetType = new NugetTypeSpecificConfigModel();
                populateSharedNuGetValues(nugetType, descriptor);
                nugetType.setMaxUniqueSnapshots(descriptor.getMaxUniqueSnapshots());
                model = nugetType;
                break;
            case Npm:
                model = new NpmTypeSpecificConfigModel();
                break;
            case Pypi:
                model = new PypiTypeSpecificConfigModel();
                break;
            case Puppet:
                model = new PuppetTypeSpecificConfigModel();
                break;
            case Vagrant:
                model = new VagrantTypeSpecificConfigModel();
                break;
            case GitLfs:
                model = new GitLfsTypeSpecificConfigModel();
                break;
            case Debian:
                DebTypeSpecificConfigModel debType = new DebTypeSpecificConfigModel();
                debType.setTrivialLayout(descriptor.isDebianTrivialLayout());
                debType.setOptionalIndexCompressionFormats(descriptor.getOptionalIndexCompressionFormats());
                debType.setDdebSupported(descriptor.isDdebSupported());
                model = debType;
                break;
            case Opkg:
                model = new OpkgTypeSpecificConfigModel();
                break;
            case YUM:
                YumTypeSpecificConfigModel yumType = new YumTypeSpecificConfigModel();
                yumType.setMetadataFolderDepth(descriptor.getYumRootDepth());
                yumType.setGroupFileNames(descriptor.getYumGroupFileNames());
                yumType.setAutoCalculateYumMetadata(descriptor.isCalculateYumMetadata());
                yumType.setEnableFileListsIndexing(descriptor.isEnableFileListsIndexing());
                model = yumType;
                break;
            case Gems:
                model = new GemsTypeSpecificConfigModel();
                break;
            case Generic:
                model = new GenericTypeSpecificConfigModel();
                break;
            case Maven:
                mavenModel = new MavenTypeSpecificConfigModel();
                break;
            case Gradle:
                mavenModel = new GradleTypeSpecificConfigModel();
                break;
            case Ivy:
                mavenModel = new IvyTypeSpecificConfigModel();
                break;
            case Pub:
                model = new PubTypeSpecificConfigModel();
                break;
            case Swift:
                model = new SwiftTypeSpecificConfigModel();
                break;
            case SBT:
                mavenModel = new SbtTypeSpecificConfigModel();
                break;
            case Terraform:
                TerraformTypeSpecificConfigModel typeSpecificConfigModel = new TerraformTypeSpecificConfigModel();
                typeSpecificConfigModel.setTerraformType(descriptor.getTerraformType());
                model = typeSpecificConfigModel;
                break;
            case TerraformBackend:
                model = new TerraformBackendTypeSpecificConfigModel();
                break;
            case BuildInfo:
                model = new BuildInfoTypeSpecificConfigModel();
                break;
        }
        if (model != null) {
            return model;
        }
        // We will get here only if our model is maven / gradle / ivy / sbt and we populate the values
        populateMavenLocalValues(mavenModel, descriptor);
        return mavenModel;
    }

    TypeSpecificConfigModel createRemoteTypeSpecific(RepoType type, HttpRepoDescriptor descriptor) {
        MavenTypeSpecificConfigModel mavenModel = null;
        TypeSpecificConfigModel model = null;
        switch (type) {
            case Maven:
                mavenModel = new MavenTypeSpecificConfigModel();
                break;
            case Gradle:
                mavenModel = new GradleTypeSpecificConfigModel();
                break;
            case Ivy:
                mavenModel = new IvyTypeSpecificConfigModel();
                break;
            case SBT:
                mavenModel = new SbtTypeSpecificConfigModel();
                break;
            case P2:
                mavenModel = new P2TypeSpecificConfigModel();
                break;
            case Debian:
                DebTypeSpecificConfigModel debType = new DebTypeSpecificConfigModel();
                debType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                debType.setDdebSupported(descriptor.isDdebSupported());
                model = debType;
                break;
            case Opkg:
                OpkgTypeSpecificConfigModel opkgType = new OpkgTypeSpecificConfigModel();
                opkgType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = opkgType;
                break;
            case Docker:
                DockerTypeSpecificConfigModel dockerType = new DockerTypeSpecificConfigModel();
                dockerType.setEnableTokenAuthentication(descriptor.isEnableTokenAuthentication());
                dockerType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                dockerType.setRetrievalCachePeriodSecs(descriptor.getRetrievalCachePeriodSecs());
                ExternalDependenciesConfig dockerExternalDependencies = descriptor.getExternalDependencies();
                if (dockerExternalDependencies != null) {
                    dockerType.setEnableForeignLayersCaching(dockerExternalDependencies.isEnabled());
                    dockerType.setExternalPatterns(dockerExternalDependencies.getPatterns());
                }
                dockerType.setBlockPushingSchema1(descriptor.isBlockPushingSchema1());
                model = dockerType;
                break;
            case NuGet:
                NugetTypeSpecificConfigModel nugetType = new NugetTypeSpecificConfigModel();
                populateSharedNuGetValues(nugetType, descriptor);
                NuGetConfiguration nuget = descriptor.getNuget();
                if (nuget != null) {
                    nugetType.setDownloadContextPath(nuget.getDownloadContextPath());
                    nugetType.setFeedContextPath(nuget.getFeedContextPath());
                    nugetType.setV3FeedUrl(nuget.getV3FeedUrl());
                    nugetType.setSymbolServerUrl(nuget.getSymbolServerUrl());
                }
                nugetType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = nugetType;
                break;
            case Npm:
                NpmTypeSpecificConfigModel npmType = new NpmTypeSpecificConfigModel();
                npmType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = npmType;
                break;
            case Pypi:
                PypiTypeSpecificConfigModel pypiType = new PypiTypeSpecificConfigModel();
                pypiType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                PypiConfiguration pyPIConfiguration = descriptor.getPypi();
                if (pyPIConfiguration != null) {
                    pypiType.setRegistryUrl(pyPIConfiguration.getPyPIRegistryUrl());
                    pypiType.setRepositorySuffix(pyPIConfiguration.getRepositorySuffix());
                }
                model = pypiType;
                break;
            case Puppet:
                PuppetTypeSpecificConfigModel puppetType = new PuppetTypeSpecificConfigModel();
                puppetType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = puppetType;
                break;
            case Chef:
                ChefTypeSpecificConfigModel chefType = new ChefTypeSpecificConfigModel();
                chefType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = chefType;
                break;
            case VCS:
                VcsTypeSpecificConfigModel vcsType = new VcsTypeSpecificConfigModel();
                populateVcsValues(vcsType, descriptor);
                model = vcsType;
                break;
            case Bower:
                BowerTypeSpecificConfigModel bowerType = new BowerTypeSpecificConfigModel();
                populateVcsValues(bowerType, descriptor);
                BowerConfiguration bowerConfiguration = descriptor.getBower();
                if (bowerConfiguration != null) {
                    bowerType.setRegistryUrl(bowerConfiguration.getBowerRegistryUrl());
                }
                model = bowerType;
                break;
            case Go:
                GoTypeSpecificConfigModel goType = new GoTypeSpecificConfigModel();
                populateVcsValues(goType, descriptor);
                goType.setMissedRetrievalCachePeriodSecs(descriptor.getMissedRetrievalCachePeriodSecs());
                model = goType;
                break;
            case CRAN:
                model = new CranTypeSpecificConfigModel();
                break;
            case Conan:
                ConanTypeSpecificConfigModel conanType = new ConanTypeSpecificConfigModel();
                conanType.setForceConanAuthentication(descriptor.isForceConanAuthentication());
                model = conanType;
                break;
            case Conda:
                model = new CondaTypeSpecificConfigModel();
                break;
            case CocoaPods:
                CocoaPodsTypeSpecificConfigModel podsType = new CocoaPodsTypeSpecificConfigModel();
                populateVcsValues(podsType, descriptor);
                CocoaPodsConfiguration podsConfiguration = descriptor.getCocoaPods();
                ExternalDependenciesConfig podsExternalDependencies = descriptor.getExternalDependencies();
                if (podsExternalDependencies != null) {
                    podsType.setEnableExternalDependencies(podsExternalDependencies.isEnabled());
                    podsType.setExternalPatterns(podsExternalDependencies.getPatterns());
                }
                if (podsConfiguration != null) {
                    podsType.setSpecsRepoUrl(podsConfiguration.getCocoaPodsSpecsRepoUrl());
                    podsType.setSpecsRepoProvider(podsConfiguration.getSpecRepoProvider());
                }
                model = podsType;
                break;
            case Gems:
                GemsTypeSpecificConfigModel gemsType = new GemsTypeSpecificConfigModel();
                gemsType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = gemsType;
                break;
            case Generic:
                GenericTypeSpecificConfigModel genericType = new GenericTypeSpecificConfigModel();
                genericType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = genericType;
                break;
            case YUM:
                YumTypeSpecificConfigModel yumType = new YumTypeSpecificConfigModel();
                yumType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                yumType.setAutoCalculateYumMetadata(null);
                yumType.setMetadataFolderDepth(null);
                yumType.setGroupFileNames(null);
                model = yumType;
                break;
            case GitLfs:
                GitLfsTypeSpecificConfigModel gitLfsType = new GitLfsTypeSpecificConfigModel();
                gitLfsType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = gitLfsType;
                break;
            case Helm:
                HelmTypeSpecificConfigModel helmType = new HelmTypeSpecificConfigModel();
                final HelmConfiguration helmConfiguration = descriptor.getHelm();
                if (helmConfiguration != null) {
                    helmType.setChartsBaseUrl(helmConfiguration.getChartsBaseUrl());
                }

                ExternalDependenciesConfig helmExternalDependencies = descriptor.getExternalDependencies();
                if (helmExternalDependencies != null) {
                    helmType.setEnableExternalDependencies(helmExternalDependencies.isEnabled());
                    helmType.setExternalPatterns(helmExternalDependencies.getPatterns());
                }

                model = helmType;
                break;
            case Alpine:
                model = new AlpineTypeSpecificConfigModel();
                break;
            case Composer:
                ComposerTypeSpecificConfigModel composerType = new ComposerTypeSpecificConfigModel();
                populateVcsValues(composerType, descriptor);
                ComposerConfiguration composerConfiguration = descriptor.getComposer();
                if (composerConfiguration != null) {
                    composerType.setRegistryUrl(composerConfiguration.getComposerRegistryUrl());
                }
                model = composerType;
                break;
            case Cargo:
                CargoTypeSpecificConfigModel cargoType = new CargoTypeSpecificConfigModel();
                GitConfiguration gitConfiguration = descriptor.getGit();
                if (gitConfiguration != null) {
                    cargoType.setRegistryUrl(gitConfiguration.getRegistryUrl());
                }
                cargoType.setAnonymousAccess(descriptor.isCargoAnonymousAccess());
                model = cargoType;
                break;
            case Pub:
                model = new PubTypeSpecificConfigModel();
                break;
            case Terraform:
                TerraformTypeSpecificConfigModel terraformTypeSpecific = new TerraformTypeSpecificConfigModel();
                populateVcsValues(terraformTypeSpecific, descriptor);
                TerraformConfiguration terraformConfiguration = descriptor.getTerraform();
                if (terraformConfiguration != null) {
                    terraformTypeSpecific.setRegistryUrl(terraformConfiguration.getTerraformRegistryUrl());
                }
                model = terraformTypeSpecific;
                break;
            case Swift:
                SwiftTypeSpecificConfigModel swiftType = new SwiftTypeSpecificConfigModel();
                ExternalDependenciesConfig externalDependencies = descriptor.getExternalDependencies();
                if (externalDependencies != null) {
                    swiftType.setEnableExternalDependencies(externalDependencies.isEnabled());
                    swiftType.setExternalPatterns(externalDependencies.getPatterns());
                }
                swiftType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
                model = swiftType;
                break;
        }
        if (model != null) {
            return model;
        }
        populateMavenRemoteValues(mavenModel, descriptor);
        return mavenModel;
    }

    private TypeSpecificConfigModel createVirtualTypeSpecific(RepoType type, VirtualRepoDescriptor descriptor) {
        TypeSpecificConfigModel typeSpecific = null;
        MavenTypeSpecificConfigModel mavenModel = null;
        switch (type) {
            case Maven:
                mavenModel = new MavenTypeSpecificConfigModel();
                mavenModel.setForceMavenAuthentication(descriptor.isForceMavenAuthentication());
                break;
            case Chef:
                ChefTypeSpecificConfigModel chef = new ChefTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    chef.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = chef;
                break;
            case Gradle:
                mavenModel = new GradleTypeSpecificConfigModel();
                break;
            case Go:
                GoTypeSpecificConfigModel goModel = new GoTypeSpecificConfigModel();
                ExternalDependenciesConfig goExternalDependencies = descriptor.getExternalDependencies();
                if (goExternalDependencies != null) {
                    goModel.setEnableExternalDependencies(goExternalDependencies.isEnabled());
                    goModel.setExternalPatterns(goExternalDependencies.getPatterns());
                }
                typeSpecific = goModel;
                break;
            case CRAN:
                CranTypeSpecificConfigModel cranTypeSpecificConfigModel = new CranTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    cranTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = cranTypeSpecificConfigModel;
                break;
            case Conan:
                ConanTypeSpecificConfigModel conanTypeSpecificConfigModel = new ConanTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    conanTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = conanTypeSpecificConfigModel;
                break;
            case Conda:
                CondaTypeSpecificConfigModel condaTypeSpecificConfigModel = new CondaTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    condaTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = condaTypeSpecificConfigModel;
                break;
            case Debian:
                DebTypeSpecificConfigModel debianTypeSpecificConfigModel = new DebTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    debianTypeSpecificConfigModel.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                    debianTypeSpecificConfigModel.setVirtualArchitectures(
                            Arrays.asList(descriptor.getDebianDefaultArchitectures().split(",")));
                    debianTypeSpecificConfigModel
                            .setOptionalIndexCompressionFormats(descriptor.getDebianOptionalIndexCompressionFormats());
                }
                debianTypeSpecificConfigModel.setDdebSupported(descriptor.isDdebSupported());
                typeSpecific = debianTypeSpecificConfigModel;
                break;
            case Helm:
                HelmTypeSpecificConfigModel helm = new HelmTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    helm.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                helm.setUseNamespaces(descriptor.isUseNamespaces());
                typeSpecific = helm;
                break;
            case Alpine:
                AlpineTypeSpecificConfigModel alpine = new AlpineTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    alpine.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = alpine;
                break;
            case Ivy:
                mavenModel = new IvyTypeSpecificConfigModel();
                break;
            case SBT:
                mavenModel = new SbtTypeSpecificConfigModel();
                break;
            case P2:
                P2TypeSpecificConfigModel p2 = new P2TypeSpecificConfigModel();
                populateVirtualP2Values(p2, descriptor);
                mavenModel = p2;
                break;
            case Gems:
                typeSpecific = new GemsTypeSpecificConfigModel();
                break;
            case GitLfs:
                typeSpecific = new GitLfsTypeSpecificConfigModel();
                break;
            case Npm:
                NpmTypeSpecificConfigModel npmType = new NpmTypeSpecificConfigModel();
                ExternalDependenciesConfig npmExternalDependencies = descriptor.getExternalDependencies();
                if (npmExternalDependencies != null) {
                    npmType.setEnableExternalDependencies(npmExternalDependencies.isEnabled());
                    if (npmExternalDependencies.getRemoteRepo() != null) {
                        npmType.setExternalRemoteRepo(npmExternalDependencies.getRemoteRepo().getKey());
                    }
                    npmType.setExternalPatterns(npmExternalDependencies.getPatterns());
                }
                if (descriptor.getVirtualCacheConfig() != null) {
                    npmType.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = npmType;
                break;
            case Bower:
                BowerTypeSpecificConfigModel bowerType = new BowerTypeSpecificConfigModel();
                ExternalDependenciesConfig bowerExternalDependencies = descriptor.getExternalDependencies();
                if (bowerExternalDependencies != null) {
                    bowerType.setEnableExternalDependencies(bowerExternalDependencies.isEnabled());
                    if (bowerExternalDependencies.getRemoteRepo() != null) {
                        bowerType.setExternalRemoteRepo(bowerExternalDependencies.getRemoteRepo().getKey());
                    }
                    bowerType.setExternalPatterns(bowerExternalDependencies.getPatterns());
                }
                typeSpecific = bowerType;
                break;
            case NuGet:
                typeSpecific = new NugetTypeSpecificConfigModel();
                populateSharedNuGetValues((NugetTypeSpecificConfigModel) typeSpecific, descriptor);
                break;
            case Pypi:
                typeSpecific = new PypiTypeSpecificConfigModel();
                break;
            case Pub:
                typeSpecific = new PubTypeSpecificConfigModel();
                break;
            case Swift:
                typeSpecific = new SwiftTypeSpecificConfigModel();
                break;
            case Puppet:
                typeSpecific = new PuppetTypeSpecificConfigModel();
                break;
            case Docker:
                DockerTypeSpecificConfigModel dockerType = new DockerTypeSpecificConfigModel();
                dockerType.setResolveDockerTagsByTimestamp(descriptor.isResolveDockerTagsByTimestamp());
                dockerType.setCachingLocalForeignLayersEnabled(descriptor.isCachingLocalForeignLayersEnabled());
                typeSpecific = dockerType;
                break;
            case YUM:
                YumTypeSpecificConfigModel yum = new YumTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    yum.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = yum;
                break;
            case Generic:
                typeSpecific = new GenericTypeSpecificConfigModel();
                break;
            case Composer:
                ComposerTypeSpecificConfigModel composer = new ComposerTypeSpecificConfigModel();
                if (descriptor.getVirtualCacheConfig() != null) {
                    composer.setVirtualRetrievalCachePeriodSecs(descriptor.getVirtualCacheConfig()
                            .getVirtualRetrievalCachePeriodSecs());
                }
                typeSpecific = composer;
                break;
            case Terraform:
                typeSpecific = new TerraformTypeSpecificConfigModel();
                break;
        }
        if (typeSpecific != null) {
            return typeSpecific;
        }
        populateMavenVirtualValues(mavenModel, descriptor);
        return mavenModel;
    }

    private void populateVcsValues(VcsTypeSpecificConfigModel vcsType, HttpRepoDescriptor descriptor) {
        vcsType.setMaxUniqueSnapshots(descriptor.getMaxUniqueSnapshots());
        VcsConfiguration vcsConf = descriptor.getVcs();
        vcsType.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
        if (vcsConf != null) {
            vcsType.setVcsType(vcsConf.getType());
            VcsGitConfiguration git = vcsConf.getGit();
            if (git != null) {
                vcsType.setGitProvider(git.getProvider());
                vcsType.setDownloadUrl(git.getDownloadUrl());
            }
        }
    }

    private void populateSharedMavenValues(MavenTypeSpecificConfigModel model, RealRepoDescriptor descriptor) {
        model.setMaxUniqueSnapshots(descriptor.getMaxUniqueSnapshots());
        model.setHandleReleases(descriptor.isHandleReleases());
        model.setHandleSnapshots(descriptor.isHandleSnapshots());
        model.setSuppressPomConsistencyChecks(descriptor.isSuppressPomConsistencyChecks());
    }

    private void populateMavenLocalValues(MavenTypeSpecificConfigModel model, LocalRepoDescriptor descriptor) {
        populateSharedMavenValues(model, descriptor);
        model.setSnapshotVersionBehavior(descriptor.getSnapshotVersionBehavior());
        model.setLocalChecksumPolicy(descriptor.getChecksumPolicyType());
    }

    private void populateMavenRemoteValues(MavenTypeSpecificConfigModel model, HttpRepoDescriptor descriptor) {
        populateSharedMavenValues(model, descriptor);
        model.setEagerlyFetchJars(descriptor.isFetchJarsEagerly());
        model.setEagerlyFetchSources(descriptor.isFetchSourcesEagerly());
        model.setRemoteChecksumPolicy(descriptor.getChecksumPolicyType());
        model.setListRemoteFolderItems(descriptor.isListRemoteFolderItems());
        model.setRejectInvalidJars(descriptor.isRejectInvalidJars());
    }

    private void populateMavenVirtualValues(MavenTypeSpecificConfigModel model, VirtualRepoDescriptor descriptor) {
        model.setPomCleanupPolicy(descriptor.getPomRepositoryReferencesCleanupPolicy());
        model.setKeyPair(descriptor.getKeyPair());
    }

    private void populateVirtualP2Values(P2TypeSpecificConfigModel model, VirtualRepoDescriptor descriptor) {
        if (descriptor.getP2() == null || descriptor.getP2().getUrls() == null) {
            return;
        }
        Map<String, String> urlToRepoKeyMap = getUrlToRepoKeyMapping(descriptor.getRepositories());
        List<P2Repo> p2Repos = Lists.newArrayList();
        descriptor.getP2().getUrls().forEach(url -> {
            if (StringUtils.startsWith(url, "local://")) {
                Optional.ofNullable(resolveLocalP2RepoFromUrl(url)).ifPresent(p2Repos::add);
            } else {
                urlToRepoKeyMap.keySet().stream()
                        .map(RepoConfigDescriptorBuilder::getUrlWithoutSubpath)
                        .filter(p2Url -> RepoConfigDescriptorBuilder.getUrlWithoutSubpath(url).equals(p2Url))
                        .findAny()
                        .ifPresent(containingUrl ->
                                p2Repos.add(new P2Repo(null, urlToRepoKeyMap.get(containingUrl), url)));
            }
        });
        model.setP2Repos(p2Repos);
    }

    P2Repo resolveLocalP2RepoFromUrl(String url) {
        String rpp = StringUtils.removeStart(url, "local://");
        //rpp = rpp.substring(0, rpp.indexOf('/'));
        RepoPath repoPath = RepoPathFactory.create(rpp);
        LocalRepoDescriptor localRepoDescriptor = centralConfig.getMutableDescriptor()
                .getLocalAndFederatedRepositoriesMap().get(repoPath.getRepoKey());
        if (localRepoDescriptor != null) {
            return new P2Repo(null, repoPath.getRepoKey(), url);
        }
        return null;
    }

    private List<PropertySetNameModel> collectPropertySets(List<PropertySet> propertySetsList) {
        return propertySetsList.stream()
                .map(PropertySetNameModel::new)
                .collect(Collectors.toList());
    }

    /**
     * Creates a mapping of url -> remote repo key to help build the P2 model (maps 'maven group' repos only)
     */
    private Map<String, String> getUrlToRepoKeyMapping(List<RepoDescriptor> descriptors) {
        ConcurrentMap<String, String> mapping = Maps.newConcurrentMap();
        descriptors.stream()
                .filter(repoDescriptor -> repoDescriptor instanceof HttpRepoDescriptor)
                .filter(repoDescriptor -> repoDescriptor.getType().isMavenGroup())
                .forEach(remoteDescriptor ->
                        mapping.put(((HttpRepoDescriptor) remoteDescriptor).getUrl(), remoteDescriptor.getKey()));
        return mapping;
    }

    private void populateSharedNuGetValues(NugetTypeSpecificConfigModel nuget, RepoBaseDescriptor descriptor) {
        nuget.setForceNugetAuthentication(descriptor.isForceNugetAuthentication());
    }

    private void populateSharedConanValues(ConanTypeSpecificConfigModel conan, RepoBaseDescriptor descriptor) {
        conan.setForceConanAuthentication(descriptor.isForceConanAuthentication());
    }

    private XrayRepoConfigModel getXrayConfig(XrayRepoConfig xrayRepoConfig) {
        XrayRepoConfigModel model = null;
        if (xrayRepoConfig != null) {
            model = new XrayRepoConfigModel();
            model.setEnabled(xrayRepoConfig.isEnabled());
            model.setDataTtl(xrayRepoConfig.getDataTtl());
        }
        return model;
    }

    private DownloadRedirectRepoConfigModel getDownloadRedirectConfig(
            DownloadRedirectRepoConfig downloadRedirectRepoConfig) {
        DownloadRedirectRepoConfigModel model = null;
        if (downloadRedirectRepoConfig != null) {
            model = new DownloadRedirectRepoConfigModel();
            model.setEnabled(downloadRedirectRepoConfig.isEnabled());
        }
        return model;
    }

    @Nullable
    private CdnRedirectRepoConfigModel getCdnRedirectConfig(@Nullable CdnRedirectRepoConfig cdnRedirectRepoConfig) {
        CdnRedirectRepoConfigModel model = null;
        if (cdnRedirectRepoConfig != null) {
            model = new CdnRedirectRepoConfigModel();
            model.setEnabled(cdnRedirectRepoConfig.isEnabled());
        }
        return model;
    }


    private void addReverseProxyConfig(AdvancedRepositoryConfigModel advanced, String repoKey) {
        CentralConfigDescriptor configDescriptor = centralConfig.getDescriptor();
        ReverseProxyDescriptor reverseProxyDescriptor = configDescriptor.getCurrentReverseProxy();
        if (reverseProxyDescriptor != null) {
            ReverseProxyRepoConfig reverseProxyRepoConfig = reverseProxyDescriptor.getReverseProxyRepoConfig(repoKey);
            advanced.setReverseProxy(reverseProxyToModel(reverseProxyRepoConfig, reverseProxyDescriptor));
        }
    }

    private ReverseProxyRepoModel reverseProxyToModel(ReverseProxyRepoConfig reverseProxy,
            ReverseProxyDescriptor reverseProxyDescriptor) {
        if (reverseProxy != null) {
            ReverseProxyRepoModel model = new ReverseProxyRepoModel();
            model.setConfigurationKey(reverseProxyDescriptor.getKey());
            model.setServerName(reverseProxy.getServerName());
            model.setServerPort(reverseProxy.getPort());
            return model;
        }
        return null;
    }

    @Autowired
    public void setCentralConfig(CentralConfigService centralConfig) {
        this.centralConfig = centralConfig;
    }

    @Autowired
    public void setAddonsManager(AddonsManager addonsManager) {
        this.addonsManager = addonsManager;
    }
}
