/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.admin.security.signingkeys;

import org.apache.commons.lang.StringUtils;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.common.keys.KeyPairService;
import org.artifactory.api.rest.keys.KeyPairModel;
import org.artifactory.descriptor.repo.KeyPairDescriptor;
import org.artifactory.keys.KeyPairsUtil;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import static org.artifactory.descriptor.repo.KeyPairDescriptor.DEFAULT_GPG_KEY;

/**
 * This Service stays as backward compatible
 * @author Chen Keinan
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class RemoveSigningKeyService implements RestService<Void> {
    private static final String PUBLIC_KEY_PARAM = "public";

    @Autowired
    AddonsManager addonsManager;
    @Autowired
    KeyPairService keyPairService;

    @Override
    public void execute(ArtifactoryRestRequest<Void> request, RestResponse response) {
        boolean isPublic = Boolean.parseBoolean(request.getQueryParamByKey(PUBLIC_KEY_PARAM));
        removeKey(isPublic);
        response.info("Key was removed");
    }

    private void removeKey(boolean isPublic) {
        KeyPairDescriptor keyPairDescriptor = keyPairService.getKeyPairByName(DEFAULT_GPG_KEY);
        KeyPairModel keyPairModel = KeyPairsUtil.toKeyPairModel(keyPairDescriptor);

        if (isPublic) {
            keyPairModel.setPublicKey(null);
        } else {
            keyPairModel.setPrivateKey(null);
        }
        if (StringUtils.isBlank(keyPairModel.getPublicKey()) &&
                StringUtils.isBlank(keyPairModel.getPrivateKey())) {
            keyPairService.deleteKeyPair(keyPairModel.getPairName());
        } else {
            keyPairService.updateKeyPair(keyPairModel);
        }
    }
}
