package org.artifactory.ui.rest.service.admin.security.ldap.ldapsettings;

import org.artifactory.api.config.CentralConfigService;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.security.SecurityDescriptor;
import org.artifactory.descriptor.security.ldap.LdapSetting;
import org.artifactory.descriptor.security.ldap.SearchPattern;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.ui.rest.model.admin.security.ldap.LdapSettingModel;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;

import static org.artifactory.ui.utils.RequestUtils.PASSWORD_PLACEHOLDER;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class GetLdapSettingsServiceTest {

    @Mock
    CentralConfigService centralConfigService;
    @Mock
    MutableCentralConfigDescriptor centralConfigDescriptor;
    @Mock
    SecurityDescriptor securityDescriptor;
    @Mock
    LdapSetting ldapSetting;

    SearchPattern searchPattern;
    GetLdapSettingsService ldapSettingsService;

    @Test
    void getLdapSettings() {
        RestResponse<LdapSettingModel> response = new ArtifactoryRestResponse<>();

        ArtifactoryRestRequest request = mock(ArtifactoryRestRequest.class);
        when(request.getPathParamByKey("id")).thenReturn("ldap");

        ldapSettingsService.execute(request, response);
        LdapSettingModel ldapSettingModel = response.getIModel();
        assertNotNull(ldapSettingModel);
        assertNotNull(ldapSettingModel.getSearch());
        assertEquals(PASSWORD_PLACEHOLDER, ldapSettingModel.getSearch().getManagerPassword());
    }

    @BeforeEach
    public void initLdap() {
        MockitoAnnotations.initMocks(this);
        when(centralConfigDescriptor.getSecurity()).thenReturn(securityDescriptor);
        when(centralConfigService.getMutableDescriptor()).thenReturn(centralConfigDescriptor);
        when(centralConfigService.getMutableDescriptor().getSecurity().getLdapSettings(
                anyString())).thenReturn(ldapSetting);
        searchPattern = new SearchPattern();
        searchPattern.setManagerPassword("SUPER_SECRET_PASSWORD");
        when(ldapSetting.getSearch()).thenReturn(searchPattern);

        ldapSettingsService = new GetLdapSettingsService(centralConfigService);
    }

    @AfterMethod
    private void noMoreCalls() {
        Mockito.verifyNoMoreInteractions(centralConfigService, centralConfigDescriptor);
    }
}