package org.artifactory.rest.resource.email;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.api.mail.MailService;
import org.artifactory.api.rest.mail.MailMessageRequest;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.util.AolUtils;
import org.artifactory.util.EmailException;
import org.jfrog.access.client.model.MessageModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import static org.artifactory.api.rest.constant.SystemRestConstants.PATH_SEND_MAIL;


@Path(PATH_SEND_MAIL)
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
@Slf4j
public class EmailResource {

    private final MailService mailService;

    @Autowired
    public EmailResource(MailService mailService) {
        this.mailService = mailService;
    }

    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed(AuthorizationService.ROLE_ADMIN)
    @POST
    public Response sendMail(MailMessageRequest mailMsg) {
        try {
            if (AolUtils.isAol()) {
                return Response.status(Response.Status.METHOD_NOT_ALLOWED)
                        .entity("Sending email not allowed on SaaS").build();
            }
            log.debug("Sending email to '{}'.", mailMsg.getRecipient());
            mailService.sendMail(new String[] {mailMsg.getRecipient()}, mailMsg.getSubject(), mailMsg.getBody());
        } catch (EmailException ex) {
            MessageModel msg =
                    new MessageModel(Response.Status.BAD_REQUEST.toString(), "sending email failed", ex.getMessage());
            return Response.status(Response.Status.BAD_REQUEST).entity(msg).type(MediaType.APPLICATION_JSON_TYPE)
                    .build();
        }
        MessageModel msg = new MessageModel(Response.Status.OK.toString(), "", "");
        return Response.ok().entity(msg).type(MediaType.APPLICATION_JSON_TYPE).build();
    }
}
