package org.artifactory.rest.resource.retention;

import org.artifactory.addon.retention.ArtifactRetentionPolicyBaseModel;
import org.artifactory.addon.retention.ArtifactRetentionPolicySearchCriteriaModel;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.common.resource.BaseResource;
import org.artifactory.rest.services.retention.ArtifactRetentionArchiveServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import static javax.ws.rs.core.MediaType.APPLICATION_JSON;
import static org.artifactory.api.rest.constant.SearchRestConstants.PARAM_POLICY_KEYS;

/**
 * @author Omri Ziv
 */
@Path("artifactRetention/archive/policies")
@Component
@RolesAllowed({AuthorizationService.ROLE_ADMIN})
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class ArtifactRetentionUIArchivePoliciesResource extends BaseResource {

    @Autowired
    private ArtifactRetentionArchiveServiceFactory artifactRetentionArchiveServiceFactory;

    @GET
    @Produces(APPLICATION_JSON)
    public Response getAllPolicies() {
        return runService(artifactRetentionArchiveServiceFactory.getAllRetentionPolicyService());
    }

    @GET
    @Path("{policyKey}")
    @Produces(APPLICATION_JSON)
    public Response getPolicy() {
        return runService(artifactRetentionArchiveServiceFactory.getRetentionPolicyService());
    }

    @POST
    @Path("{policyKey}")
    @Consumes(APPLICATION_JSON)
    public Response createPolicy(ArtifactRetentionPolicyBaseModel policy) {
        return runService(artifactRetentionArchiveServiceFactory.createRetentionPolicyService(), policy);
    }

    @DELETE
    @Path("{policyKey}")
    public Response deletePolicy() {
        return runService(artifactRetentionArchiveServiceFactory.deleteRetentionPolicyService());
    }

    @DELETE
    public Response deletePolicies(@QueryParam(PARAM_POLICY_KEYS) String policyKeys) {
        return runService(artifactRetentionArchiveServiceFactory.deleteRetentionPoliciesService(), policyKeys);
    }

    @PUT
    @Path("{policyKey}")
    @Consumes(MediaType.APPLICATION_JSON)
    @Produces(MediaType.APPLICATION_JSON)
    public Response updatePolicy(ArtifactRetentionPolicyBaseModel policy) {
        return runService(artifactRetentionArchiveServiceFactory.updateRetentionPolicyService(), policy);
    }

    @POST
    @Path("preview")
    @Consumes(APPLICATION_JSON)
    @Produces(APPLICATION_JSON)
    public Response preview(ArtifactRetentionPolicyBaseModel policy) {
        return runService(artifactRetentionArchiveServiceFactory.previewRetentionPolicyService(), policy);
    }

    @POST
    @Path("convertToAql")
    @Produces(APPLICATION_JSON)
    @Consumes(APPLICATION_JSON)
    public Response convertSearchCriteriaFormToAql(ArtifactRetentionPolicySearchCriteriaModel searchCriteriaModel) {
        return runService(artifactRetentionArchiveServiceFactory.convertPolicySearchCriteriaToAqlService(),
                searchCriteriaModel);
    }

    @POST
    @Path("{policyKey}/trigger")
    public Response triggerPolicy(@PathParam("policyKey") String policyKey) {
        return runService(artifactRetentionArchiveServiceFactory.getArtifactRetentionTriggerPolicyService());
    }

    @POST
    @Path("{policyKey}/preview")
    @Produces(APPLICATION_JSON)
    public Response previewByPolicyKey(@PathParam("policyKey") String policyKey) {
        return runService(artifactRetentionArchiveServiceFactory.getArtifactRetentionPreviewByPolicyKeyService());
    }

    @POST
    @Path("{policyKey}/stop")
    public Response stopPolicy(@PathParam("policyKey") String policyKey) {
        return runService(artifactRetentionArchiveServiceFactory.getArtifactRetentionStopPolicyService());
    }
}